/*
 * Basic temperature logging application based on swt-xy-graph.
 * 		https://code.google.com/p/swt-xy-graph/
 * 
 * The example shows a temperature diagram with data from a DataProvider class.
 * The DataProvider is a LucidControl RT-4 module. It measures temperatures by
 * using Pt1000 RTDs.
 * 
 * The Application allows polling of the temperature in adjustable intervals.
 * 
 * Please see:
 * 		http://www.lucid-control.com				(for lucidio.jar dependency)
 * 		https://code.google.com/p/swt-xy-graph/ 	(for org.eclipse.wwt.xygraph)
 * 		http://www.eclipse.org/swt/ 				(for Standard Widgets Toolkit)
 * 
 * (c) 2013 deciphe it GmbH (www.lucid-control.com)
 */

// General Imports
import java.util.TimerTask;
import java.util.Timer;
import java.util.Calendar;

// SWT Imports
import org.eclipse.swt.SWT;  
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.widgets.FileDialog;

// swt-xy-graph Imports
import org.csstudio.swt.xygraph.dataprovider.CircularBufferDataProvider;
import org.csstudio.swt.xygraph.figures.Trace;
import org.csstudio.swt.xygraph.figures.XYGraph;
import org.csstudio.swt.xygraph.figures.Trace.PointStyle;
import org.csstudio.swt.xygraph.figures.ToolbarArmedXYGraph;
import org.csstudio.swt.xygraph.figures.Trace.BaseLine;
import org.csstudio.swt.xygraph.figures.Trace.TraceType;
import org.csstudio.swt.xygraph.linearscale.Range;
import org.csstudio.swt.xygraph.undo.ZoomType;
import org.csstudio.swt.xygraph.util.XYGraphMediaFactory;

// Eclipse draw2d Imports
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.KeyEvent;
import org.eclipse.draw2d.KeyListener;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;

/*
 * Temperature logging Example
 */
public class TempLogDemoSwtXy extends Shell {
	
	
	private Timer tim;				// Update Timer
	
	String portName;				// LucidControl Port Name
	
	private int interval;			// Diagram Update Interval in milli seconds
	private int buffer;				// Data Series Circular Buffer
	
	// swt-xy-graph Diagram
	private LightweightSystem lws;
	private XYGraphTest xyGraphTest;
	private DataProvider dataProvider;
	
	// Connect Group Box Controls
	private Button buConnect;
	private Group grpConnect;
	private Button buDisconnect;
	private Label labDeviceName;
	private Text txtDeviceName;
	
	// Logging Group Box Controls
	private Button buStart;
	private Button buStop;
	private Button buClear;
	private Group grpLogging;
	
	private Button buSeries[];
	
	private Label labBuffer;
	private Text txtBuffer;
	
	private Label labInterval;
	private Text txtInterval;
	
	// Status Bar Control
	private Label labStatusBar;
	
	
	class Update extends TimerTask {
		@Override public void run() {
			boolean ret;
			double temps[] = new double[4];
			long time;
			
			// Query temperatures from RT4 module
			ret = dataProvider.getTemperatures(temps);
			
			if (ret == true) {
				// Put 4 temperatures to the chart
				time = Calendar.getInstance().getTimeInMillis();
				xyGraphTest.traceProvider[0].setCurrentYData(temps[0], time);
				xyGraphTest.traceProvider[1].setCurrentYData(temps[1], time);
				xyGraphTest.traceProvider[2].setCurrentYData(temps[2], time);
				xyGraphTest.traceProvider[3].setCurrentYData(temps[3], time);
			}
		}
	}

	/*
	 * TimerTask runs independently from the GUI tasks.
	 * 
	 * When the GUI must be updated inside a TimerTask it is necessary to 
	 * synchronize both tasks. Here you can see that this is done by calling
	 * the Update TimerTask by using syncExec. 
	 * 
	 * For SWT applications each Display provides a function timerExec. This allows
	 * to create ONE timed task per Display. The method here allows several parallel
	 * timed tasks updating the GUI.   
	 */
	class AddTimer extends TimerTask
	{
		@Override public void run()
		{
			getDisplay().syncExec(new Update());
		}
	}
	
	
	/*
	 * Constructor
	 */
	public TempLogDemoSwtXy(Display display) {
		super(display, SWT.SHELL_TRIM);
		createContents();
	}
	

	protected void createContents() {
		GridData gridData;
		
		this.setText("Temperature Logging Application");
		
		// A Grid Layout gives us high flexibility
		setLayout(new GridLayout(4, true));
		
		// Add Chart Control

		Canvas xyGraphTestCanvas = new Canvas(this, SWT.BORDER);
		xyGraphTest = new XYGraphTest();
		lws = new LightweightSystem(xyGraphTestCanvas);
		lws.setContents(xyGraphTest);
		
		gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
		gridData.horizontalSpan = 4;
		xyGraphTestCanvas.setLayoutData(gridData);
		
		// Connect Group Box
		
		grpConnect = new Group(this, SWT.None);
		grpConnect.setText("Connect Options");
		gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
		gridData.horizontalSpan = 2;
		grpConnect.setLayoutData(gridData);
		grpConnect.setLayout(new GridLayout(4, false));

		labDeviceName = new Label(grpConnect, SWT.NONE);
		gridData = new GridData();
		gridData.horizontalSpan = 1;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		labDeviceName .setLayoutData(gridData);
		labDeviceName.setText(String.format("Device Name"));
			
		txtDeviceName = new Text(grpConnect, SWT.BORDER);
		gridData = new GridData();
		gridData.horizontalSpan = 3;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		txtDeviceName.setLayoutData(gridData);
		txtDeviceName.setText("COM8");
		
		buConnect = new Button(grpConnect, SWT.PUSH);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		buConnect.setLayoutData(gridData);
		buConnect.setText("Connect");
		
		buConnect.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				boolean ret;
				
				// Connect to LucidControl RT4 Device
				try	{
					portName = txtDeviceName.getText();
					ret = dataProvider.connect(portName);
				} 
				catch (Exception e)	{
					System.out.printf("Unexpected Exception");
					ret = false;
				}
				
				if (ret == true) {
					labStatusBar.setText(String.format("Port %s opened", portName));
					buConnect.setEnabled(false);
					buDisconnect.setEnabled(true);
					txtDeviceName.setEnabled(false);
					grpLogging.setEnabled(true);
				}
				else {
					labStatusBar.setText(String.format("Port %s could not be opened", portName));
					buConnect.setEnabled(true);
					buDisconnect.setEnabled(false);
					txtDeviceName.setEnabled(true);
					grpLogging.setEnabled(false);
				}
			}
		});
		

		buDisconnect = new Button(grpConnect, SWT.PUSH);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		buDisconnect.setLayoutData(gridData);
		buDisconnect.setText("Disconnect");
		buDisconnect.setEnabled(false);
		

		buDisconnect.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				
				// Disconnect LucidControl RT-4 Device
				dataProvider.disconnect();
				
				labStatusBar.setText(String.format("Port %s closed", portName));
				buConnect.setEnabled(true);
				buDisconnect.setEnabled(false);
				txtDeviceName.setEnabled(true);
				grpLogging.setEnabled(false);
				
				buStart.setEnabled(true);
				buStop.setEnabled(false);
				txtBuffer.setEnabled(true);
				txtInterval.setEnabled(true);
				
				// Terminate Update Timer
				try {
					tim.cancel();
				}
				catch (Exception ex ){
					System.out.printf("Timer Cancel Error");
				}
			}
		});
		
		// Logging Group Box
		
		grpLogging = new Group(this, SWT.None);
		grpLogging.setText("Logging Options");
		gridData = new GridData(SWT.FILL, SWT.FILL, true, false);
		gridData.horizontalSpan = 2;
		grpLogging.setLayoutData(gridData);
		grpLogging.setLayout(new GridLayout(4, false));
		grpLogging.setEnabled(false);
		
		buStart = new Button(grpLogging, SWT.PUSH);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 1;
		buStart.setLayoutData(gridData);
		buStart.setText("Start");
		
		// Start Logging
		buStart.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				boolean parseOk = true;
				
				buStart.setEnabled(false);
				buStop.setEnabled(true);
				txtBuffer.setEnabled(false);
				txtInterval.setEnabled(false);
				
				try	{
					interval = Integer.parseInt(txtInterval.getText());
					buffer = Integer.parseInt(txtBuffer.getText());
				}
				catch (NumberFormatException ex) {
					parseOk = false;
				}
				
				if (parseOk == true) {
					xyGraphTest.traceProvider[0].setBufferSize(buffer);
					xyGraphTest.traceProvider[1].setBufferSize(buffer);
					xyGraphTest.traceProvider[2].setBufferSize(buffer);
					xyGraphTest.traceProvider[3].setBufferSize(buffer);
					tim = new Timer();
					tim.schedule(new AddTimer(), 0, interval);
				}

				
			}
		});			
		
		buStop = new Button(grpLogging, SWT.PUSH);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 1;
		buStop.setLayoutData(gridData);
		buStop.setText("Stop");
		buStop.setEnabled(false);
		
		// Stop Logging
		buStop.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				tim.cancel();
				buStop.setEnabled(false);
				buStart.setEnabled(true);
				txtBuffer.setEnabled(true);
				txtInterval.setEnabled(true);
				
			}
		});
		
		buClear = new Button(grpLogging, SWT.PUSH);
		gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		buClear.setLayoutData(gridData);
		buClear.setText("Clear");
		
		// Clear the Diagram
		buClear.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				xyGraphTest.traceProvider[0].clearTrace();
				xyGraphTest.traceProvider[1].clearTrace();
				xyGraphTest.traceProvider[2].clearTrace();
				xyGraphTest.traceProvider[3].clearTrace();
			}
		});
		
		// Create Buttons for enabling and disabling of Traces
		buSeries = new Button[4];
		for (int i = 0; i < 4; i++) {
			final int x = i;
			buSeries[x] = new Button(grpLogging, SWT.CHECK);
			buSeries[x].setText(String.format("Series %d", x));
			gridData = new GridData();
			buSeries[x].setLayoutData(gridData);
			buSeries[x].setSelection(true);

			
			buSeries[x].addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent event) {
					
					if (buSeries[x].getSelection())	{
						xyGraphTest.trace[x].setVisible(true);
					}
					else {
						xyGraphTest.trace[x].setVisible(false);
					}
				}
			});			
		}
		
		
		labBuffer = new Label(grpLogging, SWT.NONE);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		labBuffer.setLayoutData(gridData);
		labBuffer.setText("Buffer");
		
		txtBuffer = new Text(grpLogging, SWT.BORDER);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		txtBuffer.setLayoutData(gridData);
		txtBuffer.setText("1000");
		
		// Number of buffered data points in the circular buffer
		txtBuffer.addVerifyListener(new VerifyListener() {
			@Override
			public void verifyText(VerifyEvent e) {
				// TODO Auto-generated method stub
				
				Text text = (Text)e.getSource();
				final String oldS = text.getText();
				String newS = oldS.substring(0, e.start) + e.text + oldS.substring(e.end);
				int value = 0;
				
				boolean isInt = true;
				try	{
					value = Integer.parseInt(newS);
				}
				catch (NumberFormatException ex) {
					isInt = false;
				}
				
				// Do not allow more than 100.000 data points
				if (isInt == true) {
					if ((value < 0) || (value > 100000)) {
						e.doit = false;
					}
				}
				else {
					e.doit = false;
				}
			}
		});
		
		
		
		labInterval = new Label(grpLogging, SWT.NONE);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		labInterval.setLayoutData(gridData);
		labInterval.setText("Interval [ms]");
		
		txtInterval = new Text(grpLogging, SWT.BORDER);
		gridData = new GridData();
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.verticalAlignment = GridData.FILL;
		txtInterval.setLayoutData(gridData);
		txtInterval.setText("1000");
		
		// Update Interval time in milli seconds
		txtInterval.addVerifyListener(new VerifyListener() {
			@Override
			public void verifyText(VerifyEvent e) {
				// TODO Auto-generated method stub
				
				Text text = (Text)e.getSource();
				final String oldS = text.getText();
				String newS = oldS.substring(0, e.start) + e.text + oldS.substring(e.end);
				int value = 0;
				
				boolean isInt = true;
				try	{
					value = Integer.parseInt(newS);
				}
				catch (NumberFormatException ex) {
					isInt = false;
				}
				
				// Do not allow an interval shorter than 100 ms
				if (isInt == true) {
					if (value < 100) {
						e.doit = false;
					}
				}
				else {
					e.doit = false;
				}
			}
		});
		
		
		// Status bar
		
		labStatusBar = new Label(this, SWT.BORDER);
		gridData = new GridData();
		gridData.horizontalSpan = 4;
		gridData.grabExcessHorizontalSpace = true;
		gridData.horizontalAlignment = SWT.FILL;
		gridData.verticalAlignment = SWT.END;
		labStatusBar.setLayoutData(gridData);
		labStatusBar.setText("Started ...");
		
		// Create Temperature Data Provider
		dataProvider = new DataProvider(txtDeviceName.getText());
		
	}
 
	public static void main(String args[]) {
		try {
			Display display = Display.getDefault();
			
			final TempLogDemoSwtXy shell = new TempLogDemoSwtXy(display);
			shell.addListener(SWT.Close, new Listener() {
				public void handleEvent(Event event) {
					shell.dataProvider.disconnect();
				}
			});
			
			shell.setSize(1000, 800);
			shell.open();
			shell.layout();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch()) {
					display.sleep();
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
 
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
}

/*
 * Charting Class
 */
class XYGraphTest extends Figure {
	
	public Trace trace[];
	public CircularBufferDataProvider traceProvider[];
	public XYGraph xyGraph;
	private final ToolbarArmedXYGraph toolbarArmedXYGraph;
	public XYGraphTest() {

		// Setup of the Diagram
		xyGraph = new XYGraph();
		xyGraph.setTitle("Temperature Logging");
		xyGraph.setFont(XYGraphMediaFactory.getInstance().getFont(XYGraphMediaFactory.FONT_TAHOMA));
		xyGraph.primaryXAxis.setTitle("Time");
		xyGraph.primaryYAxis.setTitle("Temperature [C]");
		xyGraph.primaryXAxis.setRange(new Range(0,200));
		xyGraph.primaryXAxis.setDateEnabled(true);
		xyGraph.primaryYAxis.setAutoScale(true);
		xyGraph.primaryXAxis.setAutoScale(true);
		xyGraph.primaryXAxis.setShowMajorGrid(true);
		xyGraph.primaryYAxis.setShowMajorGrid(true);
		xyGraph.primaryXAxis.setAutoScaleThreshold(0);

		// Create four Traces with four Trace Data Providers 
		trace = new Trace[4];
		traceProvider = new CircularBufferDataProvider[4];
		
		for (int i = 0; i < 4 ; i++)
		{
			traceProvider[i] = new CircularBufferDataProvider(true);
			traceProvider[i].setBufferSize(1000);
			traceProvider[i].setUpdateDelay(100);	
			
			trace[i] = new Trace(String.format("Temp %d", i), xyGraph.primaryXAxis, xyGraph.primaryYAxis,
				traceProvider[i]);
			
			trace[i].setDataProvider(traceProvider[i]);
			trace[i].setTraceType(TraceType.SOLID_LINE);
			trace[i].setLineWidth(2);
			trace[i].setPointStyle(PointStyle.CIRCLE);
			trace[i].setPointSize(1);
			trace[i].setBaseLine(BaseLine.NEGATIVE_INFINITY);
			trace[i].setAntiAliasing(true);
			xyGraph.addTrace(trace[i]);
		}	
		
		// Activate Toolbar for the Diagram
		toolbarArmedXYGraph = new ToolbarArmedXYGraph(xyGraph);
		add(toolbarArmedXYGraph);

		//add key listener to XY-Graph. The key pressing will only be monitored when the
		//graph gains focus.
		
		xyGraph.setFocusTraversable(true);
		xyGraph.setRequestFocusEnabled(true);
		
		xyGraph.getPlotArea().addMouseListener(new MouseListener.Stub(){
			@Override
			public void mousePressed(final MouseEvent me) {
				xyGraph.requestFocus();
			}
		});	


		xyGraph.addKeyListener(new KeyListener.Stub(){
			@Override
			public void keyPressed(final KeyEvent ke) {
				if((ke.getState() == SWT.CONTROL) && (ke.keycode == 'z')){
					xyGraph.getOperationsManager().undo();
				}
				if((ke.getState() == SWT.CONTROL) && (ke.keycode == 'y')){
					xyGraph.getOperationsManager().redo();
				}
				if((ke.getState() == SWT.CONTROL) && (ke.keycode == 'x')){
					xyGraph.performAutoScale();
				}
				if((ke.getState() == SWT.CONTROL) && (ke.keycode == 's')){
					final ImageLoader loader = new ImageLoader();
					loader.data = new ImageData[]{xyGraph.getImage().getImageData()};
					  final FileDialog dialog = new FileDialog(Display.getDefault().getShells()[0], SWT.SAVE);
					    dialog
					        .setFilterNames(new String[] {"PNG Files", "All Files (*.*)" });
					    dialog.setFilterExtensions(new String[] { "*.png", "*.*" }); // Windows
					    final String path = dialog.open();
					    if((path != null) && !path.equals("")) {
                            loader.save(path, SWT.IMAGE_PNG);
                        }
				}
				if((ke.getState() == SWT.CONTROL) && (ke.keycode + 'a' -97 == 't')){
					switch (xyGraph.getZoomType()) {
					case RUBBERBAND_ZOOM:
						xyGraph.setZoomType(ZoomType.HORIZONTAL_ZOOM);
						break;
					case HORIZONTAL_ZOOM:
						xyGraph.setZoomType(ZoomType.VERTICAL_ZOOM);
						break;
					case VERTICAL_ZOOM:
						xyGraph.setZoomType(ZoomType.ZOOM_IN);
						break;
					case ZOOM_IN:
						xyGraph.setZoomType(ZoomType.ZOOM_OUT);
						break;
					case ZOOM_OUT:
						xyGraph.setZoomType(ZoomType.PANNING);
						break;
					case PANNING:
						xyGraph.setZoomType(ZoomType.NONE);
						break;
					case NONE:
						xyGraph.setZoomType(ZoomType.RUBBERBAND_ZOOM);
						break;
					default:
						break;
					}
				}
			}
		});
	}

	@Override
	protected void layout() {
		toolbarArmedXYGraph.setBounds(bounds.getCopy().shrink(5, 5));
		super.layout();
	}
}
